# Copyright (C) 2009  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


package Xyne::Common::Metalink;
use strict;
use warnings;

our $VERSION = '0.01';

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = {};
  bless ($self, $class);

  $self->{XML_VERSION} = '1.0';
  $self->{ENCODING} = 'utf-8';
  $self->{METALINK_VERSION} = '3.0';
  $self->{XMLNS} = 'http://www.metalinker.org/';

  $self->{FILES} = [];

  return $self;
}

sub add_file
{
  my ($self,$args) = @_;
  push (@{ $self->{FILES} }, $args) if (defined ($args->{'urls'}) and scalar (@{$args->{'urls'}}) > 0);
}

sub get_file_section
{
  my ($self,$args) =  @_;
  my $file_section;
  my $name;
  if (not defined ($args->{'name'}))
  {
    my $url = $args->{'urls'}->[0];
    ($name) = ($url =~ m/([^\/]+)$/);
  }
  else
  {
    $name = $args->{'name'};
  }
  my $dir = defined($args->{'dir'}) ? $args->{'dir'}.'/' : '';
  $file_section = "  <file name=\"${dir}${name}\">\n";
  
  if (defined ($args->{'identity'}))
  {
    $file_section .= "    <identity>$args->{'identity'}</identity>\n";
  }
  
  if (defined ($args->{'version'}))
  {
    $file_section .= "    <version>$args->{'version'}</version>\n";
  }
  
  if (defined ($args->{'size'}))
  {
    $file_section .= "    <size>$args->{'size'}</size>\n";
  }
  
  if (defined ($args->{'hash_type'}) and defined ($args->{'hash'}))
  {
    $file_section .= "    <verification>\n";
    $file_section .= "      <hash type=\"$args->{'hash_type'}\">$args->{'hash'}</hash>\n";
    $file_section .= "    </verification>\n";
  }
  
  my $maxconnections = (defined ($args->{'maxconnections'})) ? "maxconnections=\"$args->{'maxconnections'}\"" : '';
  $file_section .= "    <resources $maxconnections>\n";
  
  my @urls = @{$args->{'urls'}};
  
  if (defined($args->{'url_preferences'}))
  {
    my %url_preferences = %{ $args->{'url_preferences'} };
    $self->set_url_preference($_,$url_preferences{$_}) foreach (keys %url_preferences);
  }
  if (defined($args->{'url_maxconnections'}))
  {
    my %url_maxconnections = %{ $args->{'url_maxconnections'} };
    $self->set_url_maxconnections($_,$url_maxconnections{$_}) foreach (keys %url_maxconnections);
  }
  foreach my $url (@urls)
  {
    my $attributes = $self->get_url_attributes($url);
    $file_section .= "      <url $attributes>$url</url>\n";
  }
  $file_section .= "    </resources>\n";
  $file_section .= "  </file>\n";
  return $file_section;
}

sub get_metalink
{
  my ($self) = @_;
  my $xml_version = $self->{XML_VERSION};
  my $encoding = $self->{ENCODING};
  my $metalink_version = $self->{METALINK_VERSION};
  my $xmlns = $self->{XMLNS};
  my $metalink = <<"METALINK";
<?xml version="$xml_version" encoding="$encoding"?>
<metalink version="$metalink_version" xmlns="$xmlns">
<files>
METALINK

  foreach my $file (@{ $self->{FILES} })
  {
    $metalink .= $self->get_file_section($file);
  }

  $metalink .= <<METALINK;
</files>
</metalink>
METALINK
  return $metalink;
}

sub get_url_attributes
{
  my ($self,$url) = @_;
  
  my $attr;
  
  my $type = &get_url_type($url);
  $attr .= "type=\"$type\"" if defined($type);
  
  my $max = $self->get_url_maxconnections($url);
  $attr .= " maxconnections=\"$max\"" if defined($max);
  
  my $pref = $self->get_url_preference($url);
  $attr .= " preference=\"$pref\"" if defined($pref);
  
  return $attr;
}

sub get_url_maxconnections
{
  my ($self,$url) = @_;
  return $self->{URL_MAXCONNECTIONS}->{$url} if defined($self->{URL_MAXCONNECTIONS}->{$url});
  return undef;
}

sub get_url_preference
{
  my ($self,$url) = @_;
  return $self->{URL_PREFERENCES}->{$url} if defined($self->{URL_PREFERENCES}->{$url});
  return undef;
}

sub set_url_maxconnections
{
  my ($self,$url,$max) = @_;
  $self->{URL_MAXCONNECTIONS}->{$url} = $max if defined($max);
}

sub set_url_preference
{
  my ($self,$url,$pref) = @_;
  $self->{URL_PREFERENCES}->{$url} = $pref if defined($pref);
}



################################
########## CLASS SUBS ##########
################################

sub get_url_type
{
  my ($url) = @_;
  return ($url =~ m/^(.*?):\/\//) ? $1 : '';
}

1;

__END__

=head1 NAME

Xyne::Common::Metalink - A metalink class.

=head1 SYNOPSIS

  use Xyne::Common::Metalink;

=head1 DESCRIPTION

A simple class for generating metalinks.

=head2 EXPORT

None by default.

=head1 AUTHOR

Xyne <lt>xyne at archlinux dot ca<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009  Xyne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=cut
